#include <fstream>
#include <iostream>
#include <string>

#include <openbabel/mol.h>
#include <openbabel/obconversion.h>
#include <openbabel/forcefield.h>
#include <openbabel/parsmart.h>
#include <openbabel/generic.h>

using namespace std;
using namespace OpenBabel;

int main(int argc, char* argv[]) {
  // 力場の設定
  string ff = "MMFF94";
  OBForceField* pFF = OBForceField::FindForceField(ff);
  if (!pFF) {
    cerr << "指定の力場が見つかりませんでした" << endl;
    return -1;
  }
  pFF->SetLogFile(&cout);
  pFF->SetLogLevel(OBFF_LOGLVL_LOW);

  // ファイルオープン
  ifstream ifs1("protein.pdb");
  if (ifs1.fail()) {
    cerr << "ファイルが見つかりませんでした" << endl;
    return -1;
  }

  ifstream ifs2("ligand.sdf");
  if (ifs2.fail()) {
    cerr << "ファイルが見つかりませんでした" << endl;
    return -1;
  }

  // タンパク質分子の読み込み
  OBConversion conv;
  conv.SetInAndOutFormats("PDB", "PDB");
  OBMol mol;
  conv.Read(&mol, &ifs1);
  ifs1.close();

  // 低分子リガンドの読み込み
  OBConversion conv2;
  conv2.SetInAndOutFormats("SDF", "SDF");
  OBMol lig;
  conv2.Read(&lig, &ifs2);
  ifs2.close();

  // リガンドの構成原子に印を付ける
  for (auto ii = lig.BeginAtoms(); ii != lig.EndAtoms(); ii++) {
    OBGenericData* pd = new OBPairData;
    pd->SetAttribute("Ligand");
    (*ii)->SetData(pd);
  }

  mol += lig;  // タンパク質とリガンドを合わせる

  // タンパク質とリガンドのグループの設定
  OBBitVec protein(mol.NumAtoms());
  OBBitVec ligand(mol.NumAtoms());
  for (auto ii = mol.BeginAtoms(); ii != mol.EndAtoms(); ii++) {
    int ix = (*ii)->GetIdx();
    if ((*ii)->HasData("Ligand")) {
      ligand.SetBitOn(ix);
      protein.SetBitOff(ix);
    } else {
      ligand.SetBitOff(ix);
      protein.SetBitOn(ix);
    }
  }

  // タンパク質を固定する設定
  OBFFConstraints constraints;
  for (auto ii = mol.BeginAtoms(); ii != mol.EndAtoms(); ii++) {
    if (protein.BitIsSet((*ii)->GetIdx())) constraints.AddAtomConstraint((*ii)->GetIdx());
  }

  // エネルギー計算のグループの設定
  pFF->AddIntraGroup(ligand);
  pFF->AddInterGroup(ligand);
  pFF->AddInterGroups(ligand, protein);

  // セットアップ
  if (!pFF->Setup(mol, constraints)) {
    cerr << "ERROR: could not setup force field." << endl;
  }

  // リガンド構造のエネルギー極小化
  pFF->SteepestDescent(1000);
  pFF->ConjugateGradients(1000);
  pFF->GetCoordinates(mol);

  // 相互作用エネルギーの計算
  pFF->ClearGroups();
  pFF->AddInterGroups(ligand, protein);
  pFF->SetupCalculations();
  cout << endl;
  cout << "Interaction Energy = " << pFF->Energy() << " " << pFF->GetUnit() << endl;

  // 結果の出力
  ofstream ofs("out.pdb");
  conv.Write(&mol, &ofs);
  ofs.close();

  return 0;
}
